@@ -1,17 +1,22 @@
 import calendar
 import collections
 import functools
-import gzip
+import zlib
+import inspect
 import logging
 import re
-from StringIO import StringIO
 import time
-from urllib import urlencode
-import urllib2
 from xml.etree import ElementTree
 
+from evelink.thirdparty import six
+from evelink.thirdparty.six.moves import urllib
+
 _log = logging.getLogger('evelink.api')
 
+# Allows zlib.decompress to decompress gzip-compressed strings as well.
+# From zlib.h header file, not documented in Python.
+ZLIB_DECODE_AUTO = 32 + zlib.MAX_WBITS
+
 try:
     import requests
     _has_requests = True
@@ -28,13 +33,7 @@ def _clean(v):
 
 def decompress(s):
     """Decode a gzip compressed string."""
-    buf = StringIO(s)
-    f = gzip.GzipFile(fileobj=buf)
-    try:
-        return f.read()
-    finally:
-        f.close()
-        buf.close()
+    return zlib.decompress(s, ZLIB_DECODE_AUTO)
 
 
 def parse_ts(v):
@@ -213,7 +212,7 @@ class API(object):
         }
 
     def _cache_key(self, path, params):
-        sorted_params = sorted(params.iteritems())
+        sorted_params = sorted(params.items())
         # Paradoxically, Shelve doesn't like integer keys.
         return '%s-%s' % (self.CACHE_VERSION, hash((path, tuple(sorted_params))))
 
@@ -226,7 +225,7 @@ class API(object):
         """
 
         params = params or {}
-        params = dict((k, _clean(v)) for k,v in params.iteritems())
+        params = dict((k, _clean(v)) for k,v in params.items())
 
         _log.debug("Calling %s with params=%r", path, params)
         if self.api_key:
@@ -240,7 +239,7 @@ class API(object):
 
         if not cached:
             # no cached response body found, call the API for one.
-            params = urlencode(params)
+            params = urllib.parse.urlencode(params)
             full_path = "https://%s/%s.xml.aspx" % (self.base_url, path)
             response = self.send_request(full_path, params)
         else:
@@ -274,24 +273,25 @@ class API(object):
             return self.urllib2_request(full_path, params)
 
     def urllib2_request(self, full_path, params):
+        r = None
         try:
             if params:
                 # POST request
                 _log.debug("POSTing request")
-                req = urllib2.Request(full_path, data=params)
+                req = urllib.request.Request(full_path, data=params.encode())
             else:
                 # GET request
-                req = urllib2.Request(full_path)
+                req = urllib.request.Request(full_path)
                 _log.debug("GETting request")
 
             req.add_header('Accept-Encoding', 'gzip')
-            r = urllib2.urlopen(req)
-        except urllib2.HTTPError as r:
+            r = urllib.request.urlopen(req)
+        except urllib.error.HTTPError as e:
             # urllib2 handles non-2xx responses by raising an exception that
             # can also behave as a file-like object. The EVE API will return
             # non-2xx HTTP codes on API errors (since Odyssey, apparently)
-            pass
-        except urllib2.URLError as e:
+            r = e
+        except urllib.error.URLError as e:
             # TODO: Handle this better?
             raise e
 
@@ -323,6 +323,7 @@ class API(object):
             # TODO: Handle this better?
             raise e
 
+
 def auto_api(func):
     """A decorator to automatically provide an API instance.
 
@@ -339,4 +340,127 @@ def auto_api(func):
     return wrapper
 
 
+def translate_args(args, mapping=None):
+    """Translate python name variable into API parameter name."""
+    mapping = mapping if mapping else {}
+    return dict((mapping[k], v,) for k, v in args.items())
+
+# TODO: needs better name
+def get_args_and_defaults(func):
+    """Return the list of argument names and a dict of default values"""
+    specs = inspect.getargspec(func)
+    return (
+        specs.args,
+        dict(zip(specs.args[-len(specs.defaults):], specs.defaults)),
+    )
+
+
+def map_func_args(args, kw, args_names, defaults):
+    """Associate positional (*args) and key (**kw) arguments values 
+    with their argument names.
+
+    'args_names' should be the list of argument names and 'default' 
+    should be a dict associating the keyword arguments to their 
+    defaults.
+
+    Similar to inspect.getcallargs() but compatible with python 2.6.
+
+    """
+    if (len(args)+len(kw)) > len(args_names):
+        raise TypeError('Too many arguments.')
+
+    map_ = dict(zip(args_names, args))
+    for k, v in kw.items():
+        if k in map_:
+            raise TypeError(
+                "got multiple values for keyword argument '%s'" % k
+            )
+        map_[k] = v
+
+    for k, v in defaults.items():
+        map_.setdefault(k, v)
+
+    required_args = args_names[0:-len(defaults)]
+    for k in required_args:
+        if k not in map_:
+            raise TypeError("Too few arguments")
+    return map_
+
+
+class auto_call(object):
+    """A decorator to automatically provide an api response to a method.
+
+    The object the method will be bound to should have an api attribute 
+    and the method should have a keyword argument named 'api_result'.
+
+    The decorated method will have a '_request_specs' dict attribute 
+    holding:
+
+    - 'path': path of the request needs to be queried.
+
+    - 'args': method argument names.
+
+    - 'defaults': method keyword arguments and theirs default value.
+
+    - 'prop_to_param': properties of the instance the method is bound 
+    to to add as parameter of api request.
+
+    - 'map_params': dictionary associating argument name to a 
+    paramater name. They will be added to 'evelink.api._args_map' to 
+    translate argument names to parameter names.
+
+    """
+    
+    def __init__(self, path, prop_to_param=tuple(), map_params=None):
+        self.method = None
+
+        self.path = path
+        self.args = None
+        self.defaults = None
+        self.prop_to_param = prop_to_param
+        self.map_params = map_params if map_params else {}
+
+    def __call__(self, method):
+        if self.method is not None:
+            raise TypeError("This decorator method cannot be shared.")
+        self.method = method
+        
+        wrapper = self._wrapped_method()
+        
+        args, self.defaults = get_args_and_defaults(self.method)
+
+        self.args = args[1:]
+        self.args.remove('api_result')
+        self.defaults.pop('api_result')  # TODO: better exception
+
+        wrapper._request_specs = {
+            'path': self.path,
+            'args': self.args,
+            'defaults': self.defaults,
+            'prop_to_param': self.prop_to_param,
+            'map_params': self.map_params
+        }
+
+        return wrapper
+
+    def _wrapped_method(self):
+        
+        @functools.wraps(self.method)
+        def wrapper(client, *args, **kw):
+            if 'api_result' in kw:
+                return self.method(client, *args, **kw)
+                
+            args_map = map_func_args(args, kw, self.args, self.defaults)
+            for attr_name in self.prop_to_param:
+                args_map[attr_name] = getattr(client, attr_name, None)
+
+            params = translate_args(args_map, self.map_params)
+            params =  dict((k, v,) for k, v in params.items() if v is not None)
+            
+            kw['api_result'] = client.api.get(self.path, params=params)
+            return self.method(client, *args, **kw)
+
+        return wrapper
+        
+
 # vim: set ts=4 sts=4 sw=4 et:
